#include "osdmenu.h"
#include "osditem.h"
#include "menuevent.h"

time_t t;
int MaxGroup, GroupIndex;
int *CurrentGroupChannel;
int *FirstGroupChannel;
int *LastGroupChannel;

int ChannelsShown;
int ChannelsBefore;
int ChannelsAfter;

myOsdMenu::myOsdMenu() : cOsdMenu("")
{
    jumpto = false;

    if( Setup.UseSmallFont == 2 )
    {
        if(showchannelnumbers)
            cOsdMenu::SetCols(2, CHNUMWIDTH, 10, 5);
        else
            cOsdMenu::SetCols(2, 10, 5);
    }
    else
    {
        if(showchannelnumbers)
            cOsdMenu::SetCols(2, CHNUMWIDTH, 10, 6);
        else
            cOsdMenu::SetCols(2, 10, 6);
    }

    t = time(NULL);
    next = false;

    // test how many OsdItems shown on display
    // add 50 items, make a PageDown and look what item is schown
    ChannelsShown = 0;
    char buf[20]="";
    for(int i = 0; i < 50; i++)
    {
        strcpy(buf, (const char *)itoa(i));
        if( i == 0 )
            Add(new cOsdItem(buf),  true);
        else
            Add(new cOsdItem(buf),  false);
    }
    Display();

    PageDown();
    cOsdItem *tItem = Get(Current());
    ChannelsShown = atoi((const char *)tItem->Text());
    Clear();
    
    if( (ChannelsShown % 2) != 0 )
    {
        ChannelsBefore = (ChannelsShown / 2);
        ChannelsAfter = (ChannelsShown / 2);
    }
    else
    {
        ChannelsBefore = (ChannelsShown / 2);
        ChannelsAfter = (ChannelsShown / 2)-1;
    }    
    // end test
    int GroupCount = 0; // we have minimum one group
    int ChannelCount = 0;

    // Count the groups and channels
    for(cChannel *Channel = Channels.First(); Channel; Channel = Channels.Next(Channel))
    {
        if( Channel->GroupSep() )
        {
            GroupCount++;
        }else
            ChannelCount++;
    }
    
    cChannel *Channel = Channels.Last();
    ChannelCount = Channel->Number();
    
    // is the first channel a group
    bool CreateFirstGroup = false;
    Channel = Channels.First();
    if( !Channel->GroupSep() )
    {
        CreateFirstGroup = true;
        GroupCount++;
        MaxGroup++;
    }
    
    // store the group name for any channel
    char *LastGroup;
    LastGroup = new char[256];
    strcpy(LastGroup, "no group");

    // allocate memory for ChannelGroupNames
    ChannelGroupNames = new char*[ChannelCount];
    GroupNames = new char*[GroupCount];

    // create a dummy group, if we have no group as first
    GroupNames[0] = new char[ strlen(LastGroup) + 1];
    strcpy(GroupNames[0], LastGroup);
    // the next item is the first channel in this group
 
    int index = -1;
    bool FirstChannelInGroup = false;
    for(cChannel *Channel = Channels.First(); Channel; Channel = Channels.Next(Channel))
    {
        if( CreateFirstGroup )
        {
            index++;
            GroupNames[index] = new char[ strlen(LastGroup) + 1];
            strcpy(GroupNames[index], LastGroup);
            // the next item is the first channel in this group
            FirstChannelInGroup = true;
            CreateFirstGroup = false;
        }
        if( Channel->GroupSep() )   // a group separator
        {
            index++;
            // store the group name
            delete LastGroup;
            LastGroup = new char[ strlen(Channel->Name()) +1 ];
            GroupNames[index] = new char[ strlen(Channel->Name()) + 1];
            strcpy(GroupNames[index], Channel->Name());
            strcpy(LastGroup, Channel->Name());
            // the next item is the first channel in this group
            FirstChannelInGroup = true;
            //syslog(LOG_ERR, "Group: %s", GroupNames[index]);
        }
        else    // a channel
        {
            // if it is the first channel
            // set this channel as FirstGroupChannel and CurrentGroupChannel
            if( FirstChannelInGroup )
            {
                FirstChannelInGroup = false;
                FirstGroupChannel[index] = Channel->Number();
                // if first time set?
                if( CurrentGroupChannel[index] == -1 )
                    CurrentGroupChannel[index] = Channel->Number();
            }
            // if this channel is the current channel of vdr
            // set the current Channel in group and the group
            if( Channel->Number() == cDevice::CurrentChannel() )
            {
                CurrentGroupChannel[index] = Channel->Number();
                GroupIndex = index;
            }
            // override the LastGroupChannel
            LastGroupChannel[index] = Channel->Number();

            // store the group name for this channel
            ChannelGroupNames[ Channel->Number()-1 ] = new char[ strlen(LastGroup)+1 ];
            strcpy(ChannelGroupNames[ Channel->Number()-1 ], LastGroup);
        }
    }
    delete LastGroup;

    LoadSchedules(0);
}

myOsdMenu::~myOsdMenu()
{
    int index = 0;
    for(cChannel *Channel = Channels.First(); Channel; Channel = Channels.Next(Channel))
    {
        if( Channel->GroupSep() )
        {
            delete[] GroupNames[index];
            index++;
        }
        else
        {
            delete[] ChannelGroupNames[ Channel->Number()-1 ];
        }
    }

    delete[] ChannelGroupNames;
    delete[] GroupNames;
}

void myOsdMenu::LoadSchedules(int shift)
{
    t += shift * Step * 60;

    // clar all items
    Clear();

    schedules = cSchedules::Schedules(schedulesLock);
    for( cChannel *Channel = Channels.First(); Channel; Channel = Channels.Next(Channel) )
    {
        // is filtered by config?
        if( Channel->Vpid() && Channel->Apid(0) && hideradiochannels || 
             Channel->Ca() && hideencryptedchannels )
            continue;
        // only channels
        if( Channel->GroupSep() )
            continue;

        // only Channels of the current Group
        if( !(!strcmp( ChannelGroupNames[Channel->Number()-1], GroupNames[GroupIndex] )) )
            continue;

        // Hack to show the current channel in the middle of the screen
        // so do not show channels before
        // do it only if the settings middlemenuentry is set

        // only if channels in group > channels shown on display
        if( (LastGroupChannel[GroupIndex] - FirstGroupChannel[GroupIndex]) > ChannelsShown && middlemenuentry)
        {
            // at the beginning
            if( (CurrentGroupChannel[GroupIndex] - FirstGroupChannel[GroupIndex]) < ChannelsBefore )
            {
                if( (Channel->Number() - FirstGroupChannel[GroupIndex]) >= ChannelsShown )
                    continue;
            } 
            else
            {
                // in the middle
                // don't show channels from the beginning
                if( Channel->Number() < CurrentGroupChannel[GroupIndex] )
                {
                    if( (CurrentGroupChannel[GroupIndex] - Channel->Number()) > ChannelsBefore )
                        continue;
                }
                // don't show channels from the end
                if( Channel->Number() > CurrentGroupChannel[GroupIndex] )
                {
                    if( (Channel->Number() - CurrentGroupChannel[GroupIndex]) > ChannelsAfter )
                        continue;
                }
            }
            if( (LastGroupChannel[GroupIndex] - CurrentGroupChannel[GroupIndex]) < ChannelsAfter && (Channel->Number() - LastGroupChannel[GroupIndex]) >= ChannelsShown )
                continue;
        }

        const cSchedule *Schedule=schedules->GetSchedule( Channel->GetChannelID() );
        if(Schedule)
        { 
            const cEvent *Event = next ? Schedule->GetEventAround(t): Schedule->GetPresentEvent();
            Add(new myOsdItem(Event, Channel, next), Channel->Number() == CurrentGroupChannel[GroupIndex] );
        }
        else
            Add(new myOsdItem(NULL, Channel, next), Channel->Number() == CurrentGroupChannel[GroupIndex] );
    }

    sprintf(tmp, "%d:%d", bookmark/100, bookmark%100);

    SetMyTitle();

    if( switchwithok )
        SetHelp(next ? trVDR("Button$Now") : NULL, "<<", ">>", trVDR("Button$Info") );
    else
        SetHelp(bookmark ? tmp : NULL, "<<", ">>", next ? trVDR("Button$Now") : trVDR("Button$Switch"));

    Display();
}

void myOsdMenu::SetMyTitle(void)
{
    char *buffer=NULL;

    if( next )
        asprintf(&buffer, "%d/%d %s - %s", 
                GroupIndex+1, MaxGroup, GroupNames[GroupIndex], *DayDateTime(t));
    else
        asprintf(&buffer, "%d/%d %s - %s", 
                GroupIndex+1, MaxGroup, GroupNames[GroupIndex], trVDR("What's on now?"));
    SetTitle(buffer);
    free(buffer);
}
eOSState myOsdMenu::Switch()
{
    myOsdItem *item = (myOsdItem *)Get(Current());
    if(item)
    {
        const cChannel *channel = item->channel;
        if(channel && cDevice::PrimaryDevice()->SwitchChannel(channel, true) )
            return keeposd ? osContinue : osEnd;
    }
    Skins.Message(mtError, trVDR("Can't switch channel!"));
    return osContinue;
}

void myOsdMenu::GoToDay(int day)
{
    struct tm tm1,tm2;
    time_t tmp_t=time(NULL);

    localtime_r(&t,&tm1);
    localtime_r(&tmp_t,&tm2);
    tm2.tm_hour  = tm1.tm_hour;
    tm2.tm_min   = tm1.tm_min;
    tm2.tm_mday += day;

    t = mktime(&tm2);

    next = true;
    LoadSchedules(0);
}

void myOsdMenu::JumpTo(int hour, int min)
{
    struct tm tmp_tm;
    localtime_r(&t, &tmp_tm);

    if(tmp_tm.tm_hour > hour || tmp_tm.tm_min > min && tmp_tm.tm_hour >= hour)
        tmp_tm.tm_mday++;

    tmp_tm.tm_hour = hour;
    tmp_tm.tm_min  = min;

    t = mktime(&tmp_tm);

    next = true;
    LoadSchedules(0);
}

eOSState myOsdMenu::ProcessKey(eKeys Key)
{
    eOSState state;

    if(jumpto)
    {
        switch(Key)
        {
            case k0...k9:
                {
                    int number = Key-k0;
                    switch(index)
                    {
                        // first digit
                        case 0: 
                            if(number<=2)
                            {
                                hh = number * 10;
                                sprintf(tmp,"%s: %d-:--",tr("Jump to"),number);
                                index++;
                            }
                            break;
                            // second digit
                        case 1:
                            if(hh <= 10 || hh == 20 && number <= 3)
                            {
                                hh += number;
                                sprintf(tmp,"%s: %02d:--",tr("Jump to"),hh);
                                index++;
                            }
                            break;
                            // third digit
                        case 2: 
                            if(number <= 5)
                            {
                                mm = number * 10;
                                sprintf(tmp,"%s: %02d:%d-",tr("Jump to"),hh,number);
                                index++;
                            }
                            break;
                            // fourth digit
                        case 3: 
                            mm += number;
                            sprintf(tmp,"%s: %02d:%02d",tr("Jump to"),hh,mm);
                            index++;
                            break;
                        default:
                            break;
                    }
                    if(index == 4)
                    {
                        JumpTo(hh, mm);
                        jumpto = 0;
                    }
                    else
                    {
                        SetTitle(tmp);
                        Display();
                    }
                    break;
                }
            case kOk:
                JumpTo(hh, mm);
                jumpto = 0;
                break;
            case kBack:
                jumpto = 0;
                SetMyTitle();
                Display();
                break;
            default:
                break;
        }
        state = osContinue;
    }
    else
    {
        state = cOsdMenu::ProcessKey(Key);

        if(state == osUnknown)
        {
            switch(Key)
            {
                case k1...k7:
                    GoToDay(Key-k0);
                    break;
                    // toggle asking for a time to jump to
                case k0:
                    hh = mm = index =0;
                    sprintf(tmp, "%s: --:--",tr("Jump to"));
                    SetTitle(tmp);
                    Display();
                    jumpto = 1;
                    break;
                case kRed:
                    if(next)
                    {
                        next = false;
                        t = time(NULL);
                        LoadSchedules(0);
                    }
                    break;
                case kGreen:
                    next = true;
                    LoadSchedules(-1);
                    break;
                case kYellow:
                    next = true;
                    LoadSchedules(1);
                    break;
                case kBlue:
                    if( Count() )
                        return AddSubMenu(new myMenuEvent((myOsdItem*) Get(Current())));
                    break;
                case kOk:
                    if(switchwithok)
                    {
                        return Switch();
                    } else
                    {
                        if(Count())
                            return AddSubMenu(new myMenuEvent((myOsdItem*)Get(Current())));
                    }
                case kNext:
                    if( isMenuEvent )
                        return osContinue;
                    // if we switch channel groups with next/prev key do it
                    if( switchgroupkey == 1 )
                    {
                        GroupIndex++;
                        if( GroupIndex >= MaxGroup )
                            GroupIndex = 0;
                        LoadSchedules(0);
                    } else // we do a page down
                    {
                        if( !middlemenuentry ) // if middlemenuentry hack is not set we can use vdr function for page down
                            PageDown();
                        else // otherwise we must do a page down by our own
                        {
                            CurrentGroupChannel[GroupIndex] += ChannelsShown;
                            if( CurrentGroupChannel[GroupIndex] > LastGroupChannel[GroupIndex] )
                            {
                                CurrentGroupChannel[GroupIndex] = LastGroupChannel[GroupIndex];
                            }
                            LoadSchedules(0);
                        }
                    }
                    return osContinue;
                case kPrev:
                    if( isMenuEvent )
                        return osContinue;
                    // if we switch channel groups with next/prev key do it
                    if( switchgroupkey == 1 )
                    {
                        GroupIndex--;
                        if( GroupIndex < 0 )
                            GroupIndex = MaxGroup-1;
                        LoadSchedules(0);
                    } else // we do a page up
                    {
                        if( !middlemenuentry ) // if middlemenuentry hack is not set we can use vdr function for page up
                            PageUp();
                        else // otherwise we must do a page down by our own
                        {
                            CurrentGroupChannel[GroupIndex] -= ChannelsShown;
                            if( CurrentGroupChannel[GroupIndex] < FirstGroupChannel[GroupIndex] )
                            {
                                CurrentGroupChannel[GroupIndex] = FirstGroupChannel[GroupIndex];
                            }

                            LoadSchedules(0);
                        }
                    }
                    return osContinue;
                default:
                    break;
            }
        } else
        {
            switch((int)Key)
            {
                case kUp|k_Repeat:
                case kUp:
                {
                    if( isMenuEvent )
                        return osContinue;
                    if( !middlemenuentry ) // if middlemenuentry hack is not set we can let vdr kUp
                        return osContinue;
                    
                    // otherwise we 'up' our display by our own
                    CurrentGroupChannel[GroupIndex]--;
                    if( CurrentGroupChannel[GroupIndex] < FirstGroupChannel[GroupIndex] )
                        CurrentGroupChannel[GroupIndex] = LastGroupChannel[GroupIndex];

                    LoadSchedules(0);
                    return osContinue;
                }
                case kDown|k_Repeat:
                case kDown:
                {
                    if( isMenuEvent )
                        return osContinue;
                    if( !middlemenuentry ) // if middlemenuentry hack is not set we can let vdr kDown
                        return osContinue;

                    // otherwise we 'down' our display by our own
                    CurrentGroupChannel[GroupIndex]++;
                    if( CurrentGroupChannel[GroupIndex] > LastGroupChannel[GroupIndex] )
                        CurrentGroupChannel[GroupIndex] = FirstGroupChannel[GroupIndex];

                    LoadSchedules(0);
                    return osContinue;
                }
                case kLeft|k_Repeat:
                case kLeft:
                    if( switchgroupkey == 1 ) // if we not switch groups by left/right we let do vdr the rest (page up)
                    {
                        if( middlemenuentry ) // if middlemenuentry hack is set do manuel page up
                        {
                            CurrentGroupChannel[GroupIndex] -= ChannelsShown;
                            if( CurrentGroupChannel[GroupIndex] < FirstGroupChannel[GroupIndex] )
                            {
                                CurrentGroupChannel[GroupIndex] = FirstGroupChannel[GroupIndex];
                            }

                            LoadSchedules(0);
                            break;
                        }
                        
                        return osContinue;
                    }
                    // otherwise we switch the group to the left
                    GroupIndex--;
                    if( GroupIndex < 0 )
                        GroupIndex = MaxGroup-1;
                    LoadSchedules(0);
                    break;
                case kRight|k_Repeat:
                case kRight:
                    if( switchgroupkey == 1 ) // if we not switch groups by left/right we let do vdr the rest (page down)
                    {
                        if( middlemenuentry ) // if middlemenuentry hack is set do manuel page up
                        {
                            CurrentGroupChannel[GroupIndex] += ChannelsShown;
                            if( CurrentGroupChannel[GroupIndex] > LastGroupChannel[GroupIndex] )
                            {
                                CurrentGroupChannel[GroupIndex] = LastGroupChannel[GroupIndex];
                            }
                            LoadSchedules(0);
                            break;
                        }
                        
                        return osContinue;
                    }

                    // otherwise we switch the group to the right
                    GroupIndex++;
                    if( GroupIndex >= MaxGroup )
                        GroupIndex = 0;
                    LoadSchedules(0);
                    break;
                default:
                    break;
            }

        }
    }
    return state;
}
